/**
 * ListForm.java 
 * 
 * This class represent the user interface when the user clicks the 'List'
 * button in the left frame. It extends from Panel and implements an
 * ActionListener to trap the button events generated in this class. The Connection
 * class is used to communicate to the server.
 * 
 * @author xjqiu@npac.syr.edu
 */

package gui; //define package

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import client.*; //for using the Connection class 

public class ListForm extends Panel  // class definition
          implements ActionListener {

 GridBagLayout gb = new GridBagLayout();  //use a GridBagLayout
 GridBagConstraints gbc = new GridBagConstraints();  // constraints for the layout

 Color bg = new Color(255,255,255); //background and foreground colors
 Color fg = new Color(139,0,139);

 private Label l1,l2; //components for the user interface
 private CityList from,to; //list of cities
 private Button list;
 private TextArea text; //text area for displaying the data from the server 
 private StatusWin swin; // status window for displaying the status of communication
 private Connection con; //Connection object

/**
 * Constructor for the class. Builds up the user interface by placing the componenets
 * in the panel according to the constraints.
 */

 public ListForm() { 

     setLayout(gb); //sets layout, background color
     setBackground(bg);

     gbc.fill = GridBagConstraints.NONE; // sets fill as none
     gbc.anchor = GridBagConstraints.NORTHWEST; // sets anchor as NORTHWEST

     l1 = makeLabel(" From  :"); // add the From label
     addComp(l1,4,0,1,1,new Insets(-215,-35,1,1));
     from = new CityList("SYR"); // add a city list
     from.setFont(new Font("Helvetica", Font.BOLD, 12));
     from.setBackground(fg);
     from.setForeground(Color.white);
     addComp(from,5,0,1,1,new Insets(-215,0,1,1));

     l2 = makeLabel(" To :"); // add the To label
     addComp(l2,8,0,1,1,new Insets(-215,-35,1,1));
     to = new CityList("BOS"); //add a city list
     to.setFont(new Font("Helvetica", Font.BOLD, 12));
     to.setBackground(fg);
     to.setForeground(Color.white);
     addComp(to,9,0,1,1,new Insets(-215,0,1,1));

     list = new Button("      List     "); // add the List button
    // list.setBackground(fg);           // after setting the parameters
     //list.setForeground(Color.white);
     list.setFont(new Font("Helvetica", Font.BOLD, 16));
     list.addActionListener(this); //add this object itself as the listener
     addComp(list,5,20,1,2,new Insets(-165,-10,1,1));

     text = new TextArea(8,40); // add the text window to display the data
     text.setEditable(false);
     text.setBackground(new Color(66,66,99));
     text.setForeground(Color.white);
     addComp(text,4,40,1,4,new Insets(-125,-35,1,-50));

     swin = new StatusWin(); // add teh status window component
     addComp(swin,4,500,1,4,new Insets(40,-35,-450,-50));

     //write an initial string in the status window
     swin.writeMsg("Status of the client  <--> server interactions ...");
 }

/**
 * paint method called by the browser to draw a separator line.
 */

 public void paint(Graphics g) {
     g.drawLine(0,280,500,280);
 }

 /**
 * Method that actually sets the parameters and adds the component
 * to the panel. Gets the coordinates, width and height and insets
 * as well as the component object as arguments.
 */

 private void addComp
            ( Component c, int x, int y, int ht, int wt, Insets insets ) {
     gbc.gridx = x;
     gbc.gridy = y;

     gbc.gridwidth = wt;
     gbc.gridheight = ht;

     gbc.insets = insets;

     gb.setConstraints(c,gbc);
     add(c);
 }

/**
 * This method returns a Label object with the text given and with the
 * appropriate colors and fonts.
 */

 private Label makeLabel(String str) {
     Label l = new Label(str);
     l.setForeground(fg);
     l.setFont(new Font("Helvetica", Font.BOLD, 14));
     return l;
 }

/**
 * This is the ActionListener implementation. This method is called when the
 * List button in this panel is clicked. It starts the contacting the server process.
 */

 public void actionPerformed(ActionEvent e) {
     Button b = (Button) e.getSource(); //get the button object 

     String tmpstr = b.getLabel().trim(); // remove any extra spaces and return the
										  // name of the button
     if(tmpstr.equals("List")){ 
         if(text.getText() != null) {
             text.setText("");
             swin.setText("");
         }
         doList(); // start the listing process
     }
     
     b = null;
     tmpstr = null;
 }

/**
 * This method sends the query to the server and gets the response from the server and
 * adds it to the text window. This method also prints out the status message for the
 * connection in the status window. This also checks for the consistency of data
 * selection.
 */
 private void doList() {
     // checks if from and to fields are same 
     if(from.getSelectedItem().equals(to.getSelectedItem())){
         swin.writeMsg("Error ! From and To selections are same !!!");
         return;
     }
     // try to connect to the server using the Connection object
     // and prints out the status to the status window..
     swin.writeMsg("Connecting to the server....");
     con = new Connection();
     if(!con.isConnected()){
         swin.writeMsg("Error !!! while connecting to server....");
         return;
     }
     // send the action=List as the first line to the server as the
     // server is expecting an action parameter in the first line
     swin.writeMsg("Sending the query to the server...");
     if(! con.sendInfo("action=List")) {
         swin.writeMsg("Error !!! while querying the server...");
         return;
     }
     // if the action is send then get the vlues of the from and to fields
     // and send it to the server updates status window
     con.sendInfo(getValues());
     swin.writeMsg("Send complete....");

     // once everything is sent, read the response from the server
     // and send it to the textarea window
     swin.writeMsg("Getting response from server....");
     displayData(con.readInfo());
//     text.append("         "+con.readInfo());
 
     // close the connection to the server
     swin.writeMsg("Closing connection to server...");
     con.closeConnection();
 }

/**
 * This method is used to get values from the From and To choice list and formats 
 * into the appropriate format
 */

 private String getValues() {
     StringBuffer sb = new StringBuffer();
 
     sb.append("from="+from.getSelectedItem()+"##"); // get the selected items
     sb.append("to="+to.getSelectedItem()+"##");
     return sb.toString();
 }

/**
 * This method is used for formatting the data returned from the server and 
 * displaying it in the text area. It first splits the string with newline 
 * character as a delimiter and then with ## and = as delimiters and prints
 * the data onto the text area.
 */
 private void displayData(String dat) {
     Hashtable ht = new Hashtable();
     String tmp = "";
     int i = 1;
     text.append("Reservation Details for flights \n\tfrom "+ from.getSelectedItem()
                      +" to "+ to.getSelectedItem());
     StringTokenizer st = new StringTokenizer(dat,"\n");
     text.append("\nTotal # of Reservations : "+st.countTokens()+"\n");
     text.append("----------------------------------------------\n");
     while(st.hasMoreTokens()) {
         tmp = st.nextToken();
         tmp = tmp.substring(5);
         StringTokenizer st1 = new StringTokenizer(tmp,"##"); //split the line
                                                    //using '##' delimiter
         while(st1.hasMoreTokens()) {
                 // take one name=value pair and split it again to name
                 // and value strings
             StringTokenizer st2 = new StringTokenizer(st1.nextToken(),"=");
                 while(st2.hasMoreTokens()) {
                       // put the name, value string objects to the hashtable
                     ht.put(st2.nextToken(),st2.nextToken());
                 }
          } 
         // prints all the data to the text area
         text.append("\nReservation # "+ i++);
         text.append("\nDay : " + ht.get("day").toString());
         text.append("\tTime : " + ht.get("time").toString());
         text.append("\nName : " + ht.get("name").toString());
         text.append("\tPH # " + (ht.get("ph").toString()).trim());
         text.append("\n----------------------------");
         text.append("---------------");
         tmp = "";
     }
       
 }

/**
 * This method is called from the parent class for clearing or setting the default
 * options and text for the user interface 
 */

 public void clearAll() {
     from.select("SYR");
     to.select("BOS");
     swin.setText("Status of the client  <--> server interactions ...");
     text.setText("");
 }
} // end of class
