/**
 * ConnectionHandler.java
 *
 * This is the class at the server side that handles the requests to the server.
 * It gets the socket connection from the server class and opens up the input 
 * and output streams and does the processing of the request accordingly. This 
 * class also stores and retrieves data from the client to the files in the data
 * subdirectory. The main methods include handling new records and sending or 
 * listing records of data back to the client.
 *
 * @author xjqiu@npac.syr.edu
 *
 */ 

package server; // define the package

import java.io.*; // for streams and IO exceptions
import java.net.*; // for socket
import java.util.*; // for StringTokenizer and Hashtable

public class ConnectionHandler { // class definition

 private DataInputStream in; //declare an inputstream object
 private DataOutputStream out; // declare an output strea object

 private boolean errFlag = false; // error flag
 private String errMsg = ""; // error message string

 private Hashtable ht; //hashtable for storing data
 private RandomAccessFile rf; // file handle

/**
 * Constructor for the class that takes in a socket amd opens up 
 * the input and output streams. It reads the first line from the 
 * input stream to determine whether the request was for updating
 * (action=Post) the database or for retriving(action= List) data 
 * from the database.
 */ 
 public ConnectionHandler(Socket s) {
    msg("Handling connection ");
    try {
        ht = new Hashtable(); //create the Hashtable

        out = new DataOutputStream(s.getOutputStream());  //get output stream
 
        in = new DataInputStream(s.getInputStream());   //get input stream
        handleMsg(in.readUTF()); //read the first line and handle the request

    }catch(Exception e ) { //if some error occurs print it
        errMsg("Error during streaming !!! ");
        msg(e.getMessage());
    }    
 }

/**
 * The method for checking if the request was for a List or a Post. List 
 * corresponds to getting records from the database and Post corresponds to
 * posting a new record into the database. This method also calls stores the
 * query parameters or data into the hashtable and opens up the appropriate
 * files by calling the appropriate methods. Once the request is handled the
 * opened file is closed.
 */
 private void handleMsg(String msg) {

     getQueryData(); // get the rest of the information from the inputstream
     openFile(); // open the appropriate file
 
     // check if the string has an '=' in it and 
     // split the string if there is an '=' in it
     StringTokenizer st = new StringTokenizer(msg,"=");
     if(st.countTokens() <= 0) { 
         errMsg("Unrecognised Message"); // print error if no '='
     }

     //now get the split strings or tokens and get the value for action
     while (st.hasMoreTokens()) {
         if(st.nextToken().equals("action")) {
             String act = st.nextToken(); // get the value for action
             if(act.equals("Post"))
                 handleNew(); // handle new record if the value if 'Post'
             else if(act.equals("List"))
                 handleList(); // handle retrival of records if value is 'List'
             else {
                 errMsg(" Unkown option "); //else error
             }
         }else{ // error if name 'action' is not there
             errMsg(" No option specified ");
         }
     }
     closeFile(); //close the file when done
 }

/**
 * This method handles new records from the client applet. The data from the
 * client is already put into the hashtable by the getQueryData method called in
 * the previous method. This method uses the data to form a database record in
 * the format "To_city_name##name1=value1##name2=value2##...." and writes it into
 * the appropriate file opened in the previous method. Once everything is written
 * an acknowledgement is sent back to the client.
 *
 */

 private void handleNew() {
     StringBuffer sb = new StringBuffer();
     msg("handling new record"); 
               //get data from hashtable and construct the database record.
     sb.append(get("to")+"##");
     sb.append("time="+get("time")+"##");
     sb.append("day="+get("day")+"##");
     sb.append("name="+get("name")+"##");
     sb.append("ph="+get("ph")+"\n");
     writeToFile(sb.toString()); // write the record to file
     sendMsg("Updated Database"); // send acknowledgement to server
 }
 
/**
 * This method handles the list operation. It returns all the records requested
 * by the query. The query requests a from city and a to city. The from city 
 * corresponds to the file name opened in the handleMsg method and 'To' city is
 * determined here and send to the readFromFile method for getting the records.
 * Once the records are obtained, they are sent back to the client.
 */

 private void handleList() {
     String to = "";
     msg("handling list");
     to = get("to"); // get the 'To' city name
     String tmp = readFromFile(to); // get records from file
     sendMsg(tmp); // send the data back to the client
 }

/**
 * This method is used for sending messages back to the client. 
 * It uses the output stream to write messages back to the client.
 * Before writing any output to the client, it checks if any errors
 * have occurred by checking the error flag and sends out the error 
 * message present in the errMsg string instead.
 */

 private void sendMsg(String msg) {
     try {
         if(errFlag) { // check for error and send error the message
             out.writeUTF("error=1"+errMsg);
         }else { // send the message if no errors are present
             out.writeUTF(msg);
         }
         out.flush(); // flush out the contents of the stream
     }catch(Exception e) {
         errMsg("Error while sending message !!!");
         msg(e.getMessage());
     }
 }
 
/**
 * This methods reads the rest of the data from the input stream 
 * and collects them in a Hashtable for future use. A hashtable is used
 * because of it's simplicity. The string read from the stream is encoded
 * using name=value pairs and '##' delimiters.
 */

 private void getQueryData() {
     String tmp;
     try {
         tmp = in.readUTF(); // read line from the input stream
         StringTokenizer st1 = new StringTokenizer(tmp,"##"); //split the line 
                                                //using '##' delimiter
         while(st1.hasMoreTokens()) {
                 // take one name=value pair and split it again to name 
                 // and value strings
             StringTokenizer st2 = new StringTokenizer(st1.nextToken(),"=");
                 while(st2.hasMoreTokens()) {
                     // put the name, value string objects to the hashtable
                     ht.put(st2.nextToken(),st2.nextToken());
                 } 
         }
     }catch(Exception e) { 
         errMsg("Error reading from socket");
         msg(e.getMessage());
    }
 }

/**
 * get method to retrieve strings from the hashtable. It returns an empty 
 * string if no objects are present for the key given.
 */

 private String get(String key){
     String val = " ";
     if(ht.get(key) != null) { // if it is present send the string back
         val = ht.get(key).toString();
     }
     return val;
 }

/**
 * This opens up the correct data file in the data subdirectory.
 * It uses the findFile method to get the filename and opens a 
 * RandomAccessFile object in the data subdirectory.
 */

 private void openFile() {
     String filename = findFile("from"); // get the correct file name
     if(filename != null ) {
         try {
             rf = new RandomAccessFile("data/"+filename,"rw"); // open a file
         }catch(IOException e){ // print any error messages
             errMsg("Error Opening File "+ filename);
             msg(e.getMessage());
         }
     }
 }
 
/**
 * This method determines the filename with the 'From' parameter value in the
 * request sent by the client. It returns the filename string for it to be opened.
 */

 private String findFile(String f) {

     String city = (String) ht.get(f); // get the 'From' city name
     String fname = "";

     if(city != null ) { // set the appropriate file name
         if(city.equals("BOS")) 
             fname = "bos.dat";
         else if(city.equals("NYC"))
             fname = "nyc.dat";
         else if(city.equals("SYR"))
             fname = "syr.dat";
         else if(city.equals("TOR"))
             fname = "tor.dat";
     } 
     return fname;
 }

/**
 * This method writes the data to the opened file. This is synchronized so that
 * multiple simpultaneous writes takes place sequentially. It first skips the 
 * lines of files already present and appends to the end of the file
 */

 private synchronized void writeToFile(String str) {
     if(rf != null) { //if the handle is still there
         try{ // skip the lines already present 
              rf.skipBytes((new Long(rf.length())).intValue());
              rf.writeBytes(str); // write the string to the file
         }catch(Exception e) { // print any error messages present
              errMsg("Error while writing to the file !!!");
              msg(e.getMessage());
         }
     }else
         errMsg("File not open");
 }

/**
 * This methods reads the records from the opened file and sends back
 * all records where the 'To' city corresponds to the city specified in 
 * the argument. If does this by checking if the records starts with the
 * 'To' city name specified in the arguments and appends all such records 
 * into a string seperated by new lines and sends it back to the client.
 */

 private String readFromFile(String city) {

     String l = "";
     String tmp ;
     StringBuffer lines = new StringBuffer();

     if(rf != null) { // if open
         try{
             rf.seek(0); //start from the top
             while((l = rf.readLine()) != null) { // read line by line
                 if(l.startsWith(city,0)) { // check if it starts with the city
                     lines.append(l+"\r\n"); // append if it is appropriate
                 }
                 l = "";
             }
         }catch(Exception e) { //print any error messages
             errMsg("Error reading from file !!!");
             msg(e.getMessage());
         }
     }
     return lines.toString(); // returns the string
 }
 
/**
 * This method closes the open file
 */
 private void closeFile(){
     if(rf != null ) {
         try {
             rf.close(); // close the file
         }catch(Exception e) { }
     }
 }

/**
 * This method is called if any error occurs during processing. The error flag is
 * set and the error message string is set. Before sending the output back to the
 * client this flag is checked to see if any errors have occured.
 */

 private void errMsg(String str) {
     errMsg = str; // set the error message string
     errFlag = true; // set the flag as true
     msg(str); // print the message on the console window
 }

/**
 * This method prints the string on the server console window/
 */

 private void msg(String str) {
     System.out.println(str);
 }
}//end of class
